﻿using System;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Problems.QuadraticAssignment;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Tests {
  [TestClass]
  public class QAPLIBInstancesTest {
    [TestMethod]
    public void LoadAllEmbeddedInstances() {
      QuadraticAssignmentProblem qap = new QuadraticAssignmentProblem();
      StringBuilder failedInstances = new StringBuilder();
      foreach (string instance in qap.EmbeddedInstances) {
        try {
          qap.LoadEmbeddedInstance(instance);
        } catch (Exception ex) {
          failedInstances.AppendLine(instance + ": " + ex.Message);
        }
      }
      Assert.IsTrue(failedInstances.Length == 0, "Following instances failed to load: " + Environment.NewLine + failedInstances.ToString());
    }

    [TestMethod]
    public void TestReportedSolutionQuality() {
      StringBuilder failedInstances = new StringBuilder();
      QuadraticAssignmentProblem qap = new QuadraticAssignmentProblem();
      foreach (string instance in qap.EmbeddedInstances) {
        try {
          qap.LoadEmbeddedInstance(instance);
        } catch {
          Assert.Fail("Not all instances load correctly");
        }
        if (qap.BestKnownSolution != null) {
          double quality = double.NaN;
          try {
            quality = QAPEvaluator.Apply(qap.BestKnownSolution, qap.Weights, qap.Distances);
          } catch (Exception ex) {
            failedInstances.AppendLine("An unknown problem occurred evaluating solution of instance " + instance + ": " + ex.Message);
          }
          if (!quality.IsAlmost(qap.BestKnownQuality.Value)) {
            failedInstances.AppendLine(instance + ": Reported quality: " + qap.BestKnownQuality.Value.ToString() + ", evaluated fitness: " + quality.ToString() + ".");
          }
        }

      }
      Assert.IsTrue(failedInstances.Length == 0, "Following instances report divergent fitness values: " + Environment.NewLine + failedInstances.ToString());
    }
  }
}
